local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables

local preSteadyFocus = false
local watcherStarted = false
local wearingT13 = false

local ResetSteadyFocus = {
	53209--[[Chimera Shot]],
	19434--[[Aimed Shot]],
	5116--[[Concussive Shot]],
	20736--[[Distracting Shot]],
	53351--[[Kill Shot]],
	2643--[[Multi-Shot]],
	1978--[[Serpent Sting]],
	34490--[[Silencing Shot]],
	19801--[[Tranquilizing Shot]],
	82654--[[Widow Venom]],
	19386--[[Wyvern Sting]],
	109248--[[Binding Shot]],
}

local function stopPreSteadyFocus()
	if watcherStarted then
		a.print(L["Not specced into Marksman, stopping to watch for pre Steady Focus."])
		s.UnregisterModuleEvent(AddonName, "UNIT_SPELLCAST_SUCCEEDED")
		s.UnregisterModuleEvent(AddonName, "COMBAT_LOG_EVENT_UNFILTERED")
		watcherStarted = false
	end
end

local function startPreSteadyFocus()
	if not watcherStarted then
		a.print(L["Marksman detected, watching for pre Steady Focus."])
		s.RegisterModuleEvent(AddonName, "UNIT_SPELLCAST_SUCCEEDED", function (event, ...)
			local unit, _, _, _, spellID = select(1, ...)
			if unit == "player" then
				if spellID == 56641--[[Steady Shot]] then
					preSteadyFocus = true
				elseif  tContains(ResetSteadyFocus, spellID)  then
					preSteadyFocus = false
				end
			end
		end)
		s.RegisterModuleEvent(AddonName, "COMBAT_LOG_EVENT_UNFILTERED", function (event, ...)
			local _, event, _, _, sourceName, _, _, _, _, _, _, _, spellName = select(1, ...)
			if (event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REFRESH") and sourceName == UnitName("player") then
				if spellName == s.SpellName(53224--[[Steady Focus]]) then
					preSteadyFocus = false
				end
			end
		end)
		watcherStarted = true
	end
end

if s.Spec() == 2 and not watcherStarted then
	startPreSteadyFocus()
end

s.RegisterModuleEvent(AddonName, "ACTIVE_TALENT_GROUP_CHANGED", function (event, ...)
	if s.Spec() == 2 and not watcherStarted then
		startPreSteadyFocus()
	else
		stopPreSteadyFocus()
	end
end)

local HunterT13 = {
	HeadSlot = {78793, 77030, 78698}, --[[Head]]
	ShoulderSlot = {78832, 77032, 78737},  --[[Shoulder]]
	ChestSlot = {78756, 77028, 78661},  --[[Chest]]
	LegsSlot = {78804, 77031, 78709}, --[[Legs]]
	HandsSlot = {78769, 77029, 78674}, --[[Gloves]]
}

local function equipmentUpdated()
	local count = 0
	for slot, list in pairs(HunterT13) do
		if s.Equipped(list, slot) then
			count = count + 1
		end
	end
	if count >= 2 then
		if not wearingT13 then
			a.print(L["2T13 set bonus equipped."])
		end
		wearingT13 = true
	else
		if wearingT13 then
			a.print(L["No longer wearing 2T13."])
		end
		wearingT13 = false
	end
end

s.RegisterModuleEvent(AddonName, "PLAYER_EQUIPMENT_CHANGED", equipmentUpdated)
equipmentUpdated()

local HasteCDs = {
	49016--[[Unholy Frenzy]],
	80353--[[Time Warp]],
	2825--[[Bloodlust]],
	32182--[[Heroism]],
	90355--[[Ancient Hysteria]],
	26297--[[Berserking]],
	3045--[[Rapid Fire]],
	105919--[[Chronohunter]],
}

-- Determines if you will have enough Focus to cast
-- "targetSpell" when it comes off CD if you cast
-- "spellInQuestion" right now.
local function canStillCast(targetSpell, spellInQuestion)
	local timeTillTarget, _ = s.SpellCooldown(targetSpell)
	local _, gcd = s.GlobalCooldown()
	local regenRate = GetPowerRegen()
	local regenShot = 0
	if s.Casting(77767--[[Cobra Shot]], "player") then
		regenShot = 14
	end
	if s.Casting(56641--[[Steady Shot]], "player") then
		regenShot = 14
	end
	if wearingT13 then
		regenShot = regenShot * 2
	end
	if s.Buff(53224--[[Steady Focus]], "player") and regenShot > 0 then
		regenShot = regenShot + 3
	end
	regenRate = format("%.2f", regenRate)
	if timeTillTarget < gcd then
		return false
	end
	if (s.Power("player", SPELL_POWER_FOCUS) + timeTillTarget * regenRate) < (s.SpellCost(targetSpell, SPELL_POWER_FOCUS) + s.SpellCost(spellInQuestion, SPELL_POWER_FOCUS) - regenShot) then
		return false
	end
	return true
end

a.spells["Auto Shot"] = {
	ID = 75--[[Auto Shot]],
	EnemyTargetNeeded = 1,
	NotIfActive = true,
	Check = function(z)
		z.FlashColor = s.If(x.ActiveEnemy, "White", "Pink")
		return x.NoCC and not s.Shooting()
	end,
	NoGCD = true,
}

a.spells["Pet Attack"] = {
	EnemyTargetNeeded = 1,
	Override = function(z)
		if x.NoCC and x.PetAlive and s.InCombat() and x.ActiveEnemy and not s.SameTargetAsPet("target") then
			s.FlashPet(PET_ACTION_ATTACK)
		end
	end,
	FlashColor = "Pink",
	NoGCD = true,
}

a.spells["Pet Assist"] = {
	EnemyTargetNeeded = 1,
	Override = function(z)
		if s.InCombat() and not x.PetActiveEnemy then
			s.FlashPet(PET_MODE_ASSIST)
		end
	end,
	FlashColor = "Pink",
	NoGCD = true,
}

a.spells["Hunter's Mark"] = {
	ID = 1130--[[Hunter's Mark]],
	Debuff = 1130--[[Hunter's Mark]],
	EarlyRefresh = 5,
	Check = function(z)
		return not s.HasGlyph(53243--[[Glyph of Marked for Death]])
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Aspect of the Hawk"] = {
	Type = "form",
	ID = 13165--[[Aspect of the Hawk]],
	Buff = 13165--[[Aspect of the Hawk]],
	BuffUnit = "player",
	NotWhileMoving = 1,
	Check = function(z)
		return not s.Moving("player")
	end,
}

a.spells["Aspect of the Fox while moving"] = {
	Type = "form",
	ID = 82661--[[Aspect of the Fox]],
	Buff = 82661--[[Aspect of the Fox]],
	BuffUnit = "player",
	Check = function(z)
		return s.Moving("player") and not IsMounted()
	end,
}

a.spells["Explosive Trap (AoE)"] = {
	ID = 13813--[[Explosive Trap]],
	Check = function(z)
		return s.InCombat()
	end,
	FlashColor = "Purple",
	Continue = 1,
	NoRangeCheck = 1,
}

a.spells["Focus Fire at 5 stacks"] = {
	ID = 82692--[[Focus Fire]],
	RequireBuff = 19623--[[Frenzy]],
	RequireStack = 5,
	RequireBuffUnit = "pet",
	Check = function(z)
		return s.InCombat()
	end,
	NoGCD = 1,
}

a.spells["Serpent Sting if not ticking"] = {
	ID = 1978--[[Serpent Sting]],
	MyDebuff = 1978--[[Serpent Sting]],
	Run = function(z)
		z.EarlyRefresh = s.CastTime(77767--[[Cobra Shot]])
	end,
	Check = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Serpent Sting if not ticking and target HP <= 90%"] = {
	ID = 1978--[[Serpent Sting]],
	MyDebuff = 1978--[[Serpent Sting]],
	Check = function(z)
		return s.InCombat() and x.NoCC and s.Health("target") > 0 and s.HealthPercent("target") <= 90
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Serpent Sting if not ticking and TTD  >= 10"] = {
	ID = 1978--[[Serpent Sting]],
	MyDebuff = 1978--[[Serpent Sting]],
	Run = function(z)
		z.EarlyRefresh = s.CastTime(77767--[[Cobra Shot]])
	end,
	Check = function(z)
		return s.InCombat() and x.NoCC
	end,
	Run = function(z)
		if s.TimeToDie() >= 10 then
			z.Continue = 0
			z.FlashColor = "White"
		else
			z.Continue = 1
			z.FlashColor = "Pink"
		end
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Chimera Shot if target HP <= 90%"] = {
	ID = 53209--[[Chimera Shot]],
	Check = function(z)
		return s.InCombat() and x.NoCC and s.Health("target") > 0 and s.HealthPercent("target") <= 90
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Bestial Wrath if focus > 60"] = {
	ID = 19574--[[Bestial Wrath]],
	Check = function(z)
		return s.InCombat() and s.Power("player", SPELL_POWER_FOCUS) > 60
	end,
}

a.spells["Multi-Shot (AoE)"] = {
	ID = 2643--[[Multi-Shot]],
	Check = function(z)
		return s.InCombat()
	end,
	FlashColor = "Purple",
	EnemyTargetNeeded = 1,
	Continue = 1,
}

a.spells["Steady Shot (AoE)"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		local _, time = s.LastSpellTravelTime(2643--[[Multi-Shot]], "target")
		if not time then
			return false
		else
			return s.InCombat() and x.NoCC and (GetTime() - time ) < 2
		end
	end,
	FlashColor = "Purple",
	EnemyTargetNeeded = 1,
	Continue = 1,
}

a.spells["Cobra Shot (AoE)"] = {
	ID = 77767--[[Cobra Shot]],
	Check = function(z)
		local _, time = s.LastSpellTravelTime(2643--[[Multi-Shot]], "target")
		if not time then
			return false
		else
			return s.InCombat() and x.NoCC and (GetTime() - time ) < 2
		end
	end,
	FlashColor = "Purple",
	EnemyTargetNeeded = 1,
	Continue = 1,
}

a.spells["Kill Shot"] = {
	ID = 53351--[[Kill Shot]],
	Check = function(z)
		return s.InCombat() and x.NoCC and (not s.Casting(56641--[[Steady Shot]], "player") or preSteadyFocus)
	end,
	EnemyTargetNeeded = 1,
}

a.spells["A Murder of Crows"] = {
	ID = 120657--[[A Murder of Crows]],
	Check = function(z)
		return s.InCombat() and not (s.BuffDuration(HasteCDs, "player") > 0) and (not s.Casting(56641--[[Steady Shot]], "player") or preSteadyFocus)
	end,
	EnemyTargetNeeded = 1,
	NoRangeCheck = 1,
}

a.spells["A Murder of Crows with The Beast Within"] = {
	ID = 120657--[[A Murder of Crows]],
	RequireBuff = 34692--[[The Beast Within]],
	RequireBuffUnit = "player",
	Check = function(z)
		return s.InCombat()
	end,
	EnemyTargetNeeded = 1,
	NoRangeCheck = 1,
}

a.spells["Kill Command"] = {
	ID = 34026--[[Kill Command]],
	Check = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Arcane Shot with Thrill of the Hunt"] = {
	ID = 3044--[[Arcane Shot]],
	RequireBuff = 34720--[[Thrill of the Hunt]],
	RequireBuffUnit = "player",
	Check = function(z)
		return s.InCombat() and x.NoCC and (not s.Casting(56641--[[Steady Shot]], "player") or preSteadyFocus)
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Arcane Shot as focus dump when not hasted"] = {
	ID = 3044--[[Arcane Shot]],
	Check = function(z)
		return s.InCombat() and x.NoCC and canStillCast(53209--[[Chimera Shot]], 3044--[[Arcane Shot]]) and not s.Buff(HasteCDs, "player") and (not s.Casting(56641--[[Steady Shot]], "player") or preSteadyFocus)
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Arcane Shot if focus >= 69 or The Beast Within up"] = {
	ID = 3044--[[Arcane Shot]],
	Check = function(z)
		return s.InCombat() and x.NoCC and ((s.Power("player", SPELL_POWER_FOCUS) >= 69) or s.Buff(34692--[[The Beast Within]],"player"))
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Arcane Shot if focus >= 67"] = {
	ID = 3044--[[Arcane Shot]],
	Check = function(z)
		return s.InCombat() and x.NoCC and s.Power("player", SPELL_POWER_FOCUS) >= 67
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Steady Shot with Steady Focus < 3"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		return s.InCombat() and x.NoCC and preSteadyFocus and s.MyBuffDuration(53224--[[Steady Focus]], "player") < 3
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Steady Shot"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Cobra Shot"] = {
	ID = 77767--[[Cobra Shot]],
	Check = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Rapid Fire without Haste buffs and without The Beast Within"] = {
	ID = 3045--[[Rapid Fire]],
	MyBuff = 3045--[[Rapid Fire]],
	BuffUnit = "player",
	Check = function(z)
		return s.InCombat() and x.NoCC and not s.Buff(HasteCDs, "player") and not s.Buff(34692--[[The Beast Within]], "player")
	end,
	Continue = 1,
	NoGCD = 1,
	FlashColor = "Yellow",
}

a.spells["Rapid Fire without Haste buffs"] = {
	ID = 3045--[[Rapid Fire]],
	MyBuff = 3045--[[Rapid Fire]],
	BuffUnit = "player",
	Check = function(z)
		return s.InCombat() and x.NoCC and not s.Buff(HasteCDs, "player")
	end,
	Continue = 1,
	NoGCD = 1,
	FlashColor = "Yellow",
}

a.spells["Rapid Fire without Haste buffs or TTD <= 30"] = {
	ID = 3045--[[Rapid Fire]],
	MyBuff = 3045--[[Rapid Fire]],
	BuffUnit = "player",
	Check = function(z)
		return s.InCombat() and x.NoCC and (not s.Buff(HasteCDs, "player") or (s.TimeToDie() <= 30))
	end,
	Continue = 1,
	NoGCD = 1,
	FlashColor = "Yellow",
}

a.spells["Readiness after Rapid Fire"] = {
	ID = 23989--[[Readiness]],
	Check = function(z)
		return s.InCombat() and x.NoCC and s.SpellCooldown(3045--[[Rapid Fire]]) > s.GlobalCooldown()
	end,
	Continue = 1,
	NoGCD = 1,
	FlashColor = "Yellow",
}

a.spells["Aimed Shot with Master Marksman"] = {
	ID = 19434--[[Aimed Shot]],
	RequireMyBuff = 82926--[[Fire!]],
	RequireBuffUnit = "player",
	Check = function(z)
		return s.InCombat() and x.NoCC and (not s.Casting(56641--[[Steady Shot]], "player") or preSteadyFocus)
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Aimed Shot over 90% HP"] = {
	ID = 19434--[[Aimed Shot]],
	Check = function(z)
		return s.InCombat() and x.NoCC and s.Health("target") > 0 and s.HealthPercent("target") > 90 and (not s.Casting(56641--[[Steady Shot]], "player") or preSteadyFocus)
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Aimed Shot as focus dump when hasted"] = {
	ID = 19434--[[Aimed Shot]],
	RequireBuff = HasteCDs,
	RequireBuffUnit = "player",
	Check = function(z)
		return s.InCombat() and x.NoCC and canStillCast(53209--[[Chimera Shot]], 19434--[[Aimed Shot]]) and (not s.Casting(56641--[[Steady Shot]], "player") or preSteadyFocus)
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Call Pet"] = {
	ID = 883--[[Call Pet 1]],
	Check = function(z)
		return not x.PetAlive and not IsMounted()
	end,
	FlashID = {883--[[Call Pet 1]], 83242--[[Call Pet 2]], 83243--[[Call Pet 3]], 83244--[[Call Pet 4]], 83245--[[Call Pet 5]], 982--[[Revive Pet]]},
}

a.spells["Mend Pet"] = {
	ID = 136--[[Mend Pet]],
	Check = function(z)
		return x.PetAlive and s.Health("pet") > 0 and s.HealthPercent("pet") < 80
	end,
}

a.spells["Explosive Shot"] = {
	ID = 53301--[[Explosive Shot]],
	MyDebuffID = 53301--[[Explosive Shot]],
	EarlyRefresh = 1.9,
	Check = function(z)
		return s.InCombat() and x.NoCC and not s.SpellOrAuraDelay(53301--[[Explosive Shot]], "target", true)
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Black Arrow"] = {
	ID = 3674--[[Black Arrow]],
	MyDebuffID = 3674--[[Black Arrow]],
	Check = function(z)
		return s.InCombat() and x.NoCC
	end,
	Run = function(z)
		if s.TimeToDie() >= 8 then
			z.Continue = 0
			z.FlashColor = "White"
		else
			z.Continue = 1
			z.FlashColor = "Pink"
		end
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Disengage when movement impaired"] = {
	ID = 781--[[Disengage]],
	Check = function(z)
		return s.MovementImpaired("player") and s.HasTalent(109215--[[Posthaste]])
	end,
	FlashCOlor = "Yellow",
	NoGCD = true,
}

a.spells["Silencing Shot"] = {
	ID = 34490--[[Silencing Shot]],
	Debuff = 34490--[[Silencing Shot]],
	EnemyTargetNeeded = 1,
	Interrupt = 1,
	NoGCD = 1,
	FlashColor = "Aqua",
}
